"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFeatureValueObject = exports.getFeatureMainValue = exports.getFeatureMainProperty = exports.parseFeatureIdentifier = exports.generateFeaturesConfig = exports.loadFeaturesJsonFromDisk = exports.loadFeaturesJson = exports.generateContainerEnvs = exports.getFeatureLayers = exports.getContainerFeaturesBaseDockerFile = exports.getSourceInfoString = exports.getContainerFeaturesFolder = exports.multiStageBuildExploration = exports.collapseFeaturesConfig = exports.isV2FeaturesConfigRepresentation = void 0;
const jsonc = __importStar(require("jsonc-parser"));
const path = __importStar(require("path"));
const URL = __importStar(require("url"));
const tar = __importStar(require("tar"));
const fs_1 = require("fs");
const pfs_1 = require("../spec-utils/pfs");
const log_1 = require("../spec-utils/log");
const httpRequest_1 = require("../spec-utils/httpRequest");
const containerFeaturesOrder_1 = require("./containerFeaturesOrder");
const V1_ASSET_NAME = 'devcontainer-features.tgz';
// There are two ways to represent features in the user's devcontainer.json
// v1 - Object Notation
// v2 - Array Notation
// This returns true if the v2 notation is used.
function isV2FeaturesConfigRepresentation(features) {
    return Array.isArray(features) && !features.some(f => f === undefined);
}
exports.isV2FeaturesConfigRepresentation = isV2FeaturesConfigRepresentation;
function collapseFeaturesConfig(original) {
    const collapsed = {
        allFeatures: original.featureSets
            .map(fSet => fSet.features)
            .flat()
    };
    return collapsed;
}
exports.collapseFeaturesConfig = collapseFeaturesConfig;
exports.multiStageBuildExploration = false;
// Counter to ensure that no two folders are the same even if we are executing the same feature multiple times.
let counter = 1;
function getCounter() {
    return counter++;
}
const isTsnode = path.basename(process.argv[0]) === 'ts-node' || process.argv.indexOf('ts-node/register') !== -1;
function getContainerFeaturesFolder(_extensionPath) {
    if (isTsnode) {
        return path.join(require.resolve('vscode-dev-containers/package.json'), '..', 'container-features');
    }
    const distFolder = typeof _extensionPath === 'string' ? path.join(_extensionPath, 'dist') : _extensionPath.distFolder;
    return path.join(distFolder, 'node_modules', 'vscode-dev-containers', 'container-features');
}
exports.getContainerFeaturesFolder = getContainerFeaturesFolder;
// Take a SourceInformation and condense it down into a single string
// Useful for calculating a unique build folder name for a given featureSet.
function getSourceInfoString(srcInfo) {
    const { type } = srcInfo;
    switch (type) {
        case 'local-cache':
            return 'local-cache-' + getCounter();
        case 'direct-tarball':
            return srcInfo.tarballUri + getCounter();
        case 'github-repo':
            return `github-${srcInfo.owner}-${srcInfo.repo}-${srcInfo.isLatest ? 'latest' : srcInfo.tag}-${getCounter()}`;
        case 'file-path':
            return srcInfo.filePath + '-' + getCounter();
    }
}
exports.getSourceInfoString = getSourceInfoString;
// TODO: Move to node layer.
function getContainerFeaturesBaseDockerFile() {
    return `
#{featureBuildStages}

#{nonBuildKitFeatureContentFallback}

FROM $_DEV_CONTAINERS_BASE_IMAGE

USER root

COPY --from=dev_containers_feature_content_source {contentSourceRootPath} /tmp/build-features/

#{featureLayer}

#{copyFeatureBuildStages}

#{containerEnv}

ARG _DEV_CONTAINERS_IMAGE_USER=root
USER $_DEV_CONTAINERS_IMAGE_USER
`;
}
exports.getContainerFeaturesBaseDockerFile = getContainerFeaturesBaseDockerFile;
function getFeatureLayers(featuresConfig) {
    let result = '';
    // Features version 1
    const folders = (featuresConfig.featureSets || []).filter(y => y.internalVersion !== '2').map(x => x.features[0].consecutiveId);
    folders.forEach(folder => {
        result += `RUN cd /tmp/build-features/${folder} \\
&& chmod +x ./install.sh \\
&& ./install.sh

`;
    });
    // Features version 2
    featuresConfig.featureSets.filter(y => y.internalVersion === '2').forEach(featureSet => {
        featureSet.features.forEach(feature => {
            result += generateContainerEnvs(feature);
            result += `
				
RUN cd /tmp/build-features/${feature.consecutiveId} \\
&& export $(cat devcontainer-features.env | xargs) \\
&& chmod +x ./${feature.runParams} \\
&& ./${feature.runParams}

`;
        });
    });
    return result;
}
exports.getFeatureLayers = getFeatureLayers;
// Features version two export their environment variables as part of the Dockerfile to make them available to subsequent features.
function generateContainerEnvs(feature) {
    let result = '';
    if (!feature.containerEnv) {
        return result;
    }
    let keys = Object.keys(feature.containerEnv);
    result = keys.map(k => `ENV ${k}=${feature.containerEnv[k]}`).join('\n');
    return result;
}
exports.generateContainerEnvs = generateContainerEnvs;
const allowedFeatureIdRegex = new RegExp('^[a-zA-Z0-9_-]*$');
// Parses a declared feature in user's devcontainer file into
// a usable URI to download remote features.
// RETURNS
// {
//  "id",              <----- The ID of the feature in the feature set.
//  sourceInformation  <----- Source information (is this locally cached, a GitHub remote feature, etc..), including tarballUri if applicable.
// }
//
const cleanupIterationFetchAndMerge = async (tempTarballPath, output) => {
    // Non-fatal, will just get overwritten if we don't do the cleaned up.
    try {
        await (0, pfs_1.rmLocal)(tempTarballPath, { force: true });
    }
    catch (e) {
        output.write(`Didn't remove temporary tarball from disk with caught exception: ${e === null || e === void 0 ? void 0 : e.Message} `, log_1.LogLevel.Trace);
    }
};
function getRequestHeaders(sourceInformation, env, output) {
    let headers = {
        'user-agent': 'devcontainer'
    };
    const isGitHubUri = (srcInfo) => {
        const uri = srcInfo.tarballUri;
        return uri.startsWith('https://github.com') || uri.startsWith('https://api.github.com');
    };
    if (sourceInformation.type === 'github-repo' || (sourceInformation.type === 'direct-tarball' && isGitHubUri(sourceInformation))) {
        const githubToken = env['GITHUB_TOKEN'];
        if (githubToken) {
            output.write('Using environment GITHUB_TOKEN.');
            headers.Authorization = `Bearer ${githubToken}`;
        }
        else {
            output.write('No environment GITHUB_TOKEN available.');
        }
    }
    return headers;
}
async function askGitHubApiForTarballUri(sourceInformation, feature, headers, output) {
    const options = {
        type: 'GET',
        url: sourceInformation.apiUri,
        headers
    };
    const apiInfo = JSON.parse(((await (0, httpRequest_1.request)(options, output)).toString()));
    if (apiInfo) {
        const asset = apiInfo.assets.find(a => a.name === `${feature.id}.tgz`) // v2
            || apiInfo.assets.find(a => a.name === V1_ASSET_NAME) // v1
            || undefined;
        if (asset && asset.url) {
            output.write(`Found url to fetch release artifact '${asset.name}'. Asset of size ${asset.size} has been downloaded ${asset.download_count} times and was last updated at ${asset.updated_at}`);
            return asset.url;
        }
        else {
            output.write('Unable to fetch release artifact URI from GitHub API', log_1.LogLevel.Error);
            return undefined;
        }
    }
    return undefined;
}
async function loadFeaturesJson(jsonBuffer, filePath, output) {
    if (jsonBuffer.length === 0) {
        output.write('Parsed featureSet is empty.', log_1.LogLevel.Error);
        return undefined;
    }
    const featureSet = jsonc.parse(jsonBuffer.toString());
    if (!(featureSet === null || featureSet === void 0 ? void 0 : featureSet.features) || featureSet.features.length === 0) {
        output.write('Parsed featureSet contains no features.', log_1.LogLevel.Error);
        return undefined;
    }
    output.write(`Loaded ${filePath}, which declares ${featureSet.features.length} features and ${(!!featureSet.sourceInformation) ? 'contains' : 'does not contain'} explicit source info.`, log_1.LogLevel.Trace);
    return updateFromOldProperties(featureSet);
}
exports.loadFeaturesJson = loadFeaturesJson;
async function loadFeaturesJsonFromDisk(pathToDirectory, output) {
    const filePath = path.join(pathToDirectory, 'devcontainer-features.json');
    const jsonBuffer = await (0, pfs_1.readLocalFile)(filePath);
    return loadFeaturesJson(jsonBuffer, filePath, output);
}
exports.loadFeaturesJsonFromDisk = loadFeaturesJsonFromDisk;
function updateFromOldProperties(original) {
    // https://github.com/microsoft/dev-container-spec/issues/1
    if (!original.features.find(f => f.extensions || f.settings)) {
        return original;
    }
    return {
        ...original,
        features: original.features.map(f => {
            if (!(f.extensions || f.settings)) {
                return f;
            }
            const copy = { ...f };
            const customizations = copy.customizations || (copy.customizations = {});
            const vscode = customizations.vscode || (customizations.vscode = {});
            if (copy.extensions) {
                vscode.extensions = (vscode.extensions || []).concat(copy.extensions);
                delete copy.extensions;
            }
            if (copy.settings) {
                vscode.settings = {
                    ...copy.settings,
                    ...(vscode.settings || {}),
                };
                delete copy.settings;
            }
            return copy;
        }),
    };
}
// Generate a base featuresConfig object with the set of locally-cached features, 
// as well as downloading and merging in remote feature definitions.
async function generateFeaturesConfig(params, dstFolder, config, imageLabelDetails, getLocalFolder) {
    const { output } = params;
    if (!config.features) {
        return undefined;
    }
    if (!imageLabelDetails) {
        return undefined;
    }
    // Converts from object notation to array notation.
    const userFeatures = convertOldFeatures(params, config);
    if (!userFeatures) {
        return undefined;
    }
    // Create the featuresConfig object.
    // Initialize the featureSets object, and stash the dstFolder on the object for use later.
    let featuresConfig = {
        featureSets: [],
        dstFolder
    };
    // load local cache of features;
    // TODO: Update so that cached features are always version 2
    let locallyCachedFeatureSet = await loadFeaturesJsonFromDisk(getLocalFolder(params.extensionPath), output); // TODO: Pass dist folder instead to also work with the devcontainer.json support package.
    if (!locallyCachedFeatureSet) {
        output.write('Failed to load locally cached features', log_1.LogLevel.Error);
        return undefined;
    }
    // Read features and get the type.
    output.write('--- Processing User Features ----', log_1.LogLevel.Trace);
    featuresConfig = await processUserFeatures(params.output, userFeatures, featuresConfig);
    // Fetch features and get version information
    output.write('--- Fetching User Features ----', log_1.LogLevel.Trace);
    await fetchFeatures(params, featuresConfig, locallyCachedFeatureSet, dstFolder);
    const ordererdFeatures = (0, containerFeaturesOrder_1.computeFeatureInstallationOrder)(config, featuresConfig.featureSets);
    output.write('--- Computed order ----', log_1.LogLevel.Trace);
    for (const feature of ordererdFeatures) {
        output.write(`${feature.features[0].id}`, log_1.LogLevel.Trace);
    }
    featuresConfig.featureSets = ordererdFeatures;
    return featuresConfig;
}
exports.generateFeaturesConfig = generateFeaturesConfig;
// Convert features from object syntax to array syntax
function convertOldFeatures(params, config) {
    params.output.write('');
    let userFeatures = [];
    if (Array.isArray(config.features)) {
        userFeatures = config.features;
    }
    else {
        if (!config.features || !Object.keys(config.features).length) {
            return undefined;
        }
        for (const userFeatureKey of Object.keys(config.features)) {
            const userFeatureValue = config.features[userFeatureKey];
            if (userFeatureKey) {
                let feature = {
                    id: userFeatureKey,
                    options: userFeatureValue
                };
                userFeatures.push(feature);
            }
        }
    }
    return userFeatures;
}
// Process features contained in devcontainer.json
// Creates one feature set per feature to aid in support of the previous structure.
async function processUserFeatures(output, userFeatures, featuresConfig) {
    userFeatures.forEach(userFeature => {
        const newFeatureSet = parseFeatureIdentifier(output, userFeature);
        if (newFeatureSet) {
            featuresConfig.featureSets.push(newFeatureSet);
        }
    });
    return featuresConfig;
}
function parseFeatureIdentifier(output, userFeature) {
    // A identifier takes this form:
    //      (0)  <feature>
    //      (1)  <publisher>/<feature-set>/<feature>@version
    //      (2)  https://<../URI/..>/devcontainer-features.tgz#<feature>
    //      (3) ./<local-path>#<feature>  -or-  ../<local-path>#<feature>  -or-   /<local-path>#<feature>
    // 
    //  (0) This is a locally cached feature.
    //
    //  (1) Our "registry" is backed by GitHub public repositories (or repos visible with the environment's GITHUB_TOKEN).
    //      Say organization 'octocat' has a repo titled 'myfeatures' with a set of feature definitions.
    //      One of the [1..n] features in this repo has an id of 'helloworld'.
    //
    //      eg: octocat/myfeatures/helloworld
    //
    //      The above example assumes the 'latest' GitHub release, and internally will 
    //      fetch the devcontainer-features.tgz artifact from that release.
    //      To specify a certain release tag, append the tag with an @ symbol
    //
    //      eg: octocat/myfeatures/helloworld@v0.0.2
    //
    //  (2) A fully-qualified https URI to a devcontainer-features.tgz file can be provided instead
    //      of a using the GitHub registry "shorthand". Note this is identified by a
    //      s.StartsWith("https://" ||  "http://").
    //
    //      eg: https://example.com/../../devcontainer-features.tgz#helloworld
    //
    //  (3) This is a local path to a directory on disk following the expected file convention
    //      The path can either be:
    //          -  a relative file path to the .devcontainer file (prepended by a ./  or ../)
    //          -  an absolute file path (prepended by a /)
    //
    //      No version can be provided, as the directory is copied 'as is' and is inherently taking the 'latest'
    output.write(`* Processing feature: ${userFeature.id}`);
    // cached feature
    if (!userFeature.id.includes('/') && !userFeature.id.includes('\\')) {
        output.write(`Cached feature found.`);
        let feat = {
            id: userFeature.id,
            name: userFeature.id,
            value: userFeature.options,
            included: true,
        };
        let newFeaturesSet = {
            sourceInformation: {
                type: 'local-cache',
            },
            features: [feat],
        };
        return newFeaturesSet;
    }
    // remote tar file
    if (userFeature.id.startsWith('http://') || userFeature.id.startsWith('https://')) {
        output.write(`Remote tar file found.`);
        let input = userFeature.id.replace(/\/+$/, '');
        const featureIdDelimiter = input.lastIndexOf('#');
        const id = input.substring(featureIdDelimiter + 1);
        if (id === '' || !allowedFeatureIdRegex.test(id)) {
            output.write(`Parse error. Specify a feature id with alphanumeric, dash, or underscore characters. Provided: ${id}.`, log_1.LogLevel.Error);
            return undefined;
        }
        const tarballUri = new URL.URL(input.substring(0, featureIdDelimiter)).toString();
        let feat = {
            id: id,
            name: userFeature.id,
            value: userFeature.options,
            included: true,
        };
        let newFeaturesSet = {
            sourceInformation: {
                type: 'direct-tarball',
                tarballUri: tarballUri
            },
            features: [feat],
        };
        return newFeaturesSet;
    }
    // local disk
    const userFeaturePath = path.parse(userFeature.id);
    // If its a valid path
    if (userFeature.id.startsWith('./') || userFeature.id.startsWith('../') || (userFeaturePath && path.isAbsolute(userFeature.id))) {
        //if (userFeaturePath && ((path.isAbsolute(userFeature.id) && existsSync(userFeature.id)) || !path.isAbsolute(userFeature.id))) {
        output.write(`Local disk feature.`);
        const filePath = userFeature.id;
        const id = userFeaturePath.name;
        const isRelative = !path.isAbsolute(userFeature.id);
        let feat = {
            id: id,
            name: userFeature.id,
            value: userFeature.options,
            included: true,
        };
        let newFeaturesSet = {
            sourceInformation: {
                type: 'file-path',
                filePath,
                isRelative: isRelative
            },
            features: [feat],
        };
        return newFeaturesSet;
    }
    output.write(`Github feature.`);
    // Github repository source.
    let version = 'latest';
    let splitOnAt = userFeature.id.split('@');
    if (splitOnAt.length > 2) {
        output.write(`Parse error. Use the '@' symbol only to designate a version tag.`, log_1.LogLevel.Error);
        return undefined;
    }
    if (splitOnAt.length === 2) {
        output.write(`[${userFeature.id}] has version ${splitOnAt[1]}`, log_1.LogLevel.Trace);
        version = splitOnAt[1];
    }
    // Remaining info must be in the first part of the split.
    const featureBlob = splitOnAt[0];
    const splitOnSlash = featureBlob.split('/');
    // We expect all GitHub/registry features to follow the triple slash pattern at this point
    //  eg: <publisher>/<feature-set>/<feature>
    if (splitOnSlash.length !== 3 || splitOnSlash.some(x => x === '') || !allowedFeatureIdRegex.test(splitOnSlash[2])) {
        output.write(`Invalid parse for GitHub/registry feature identifier. Follow format: '<publisher>/<feature-set>/<feature>'`, log_1.LogLevel.Error);
        return undefined;
    }
    const owner = splitOnSlash[0];
    const repo = splitOnSlash[1];
    const id = splitOnSlash[2];
    let feat = {
        id: id,
        name: userFeature.id,
        value: userFeature.options,
        included: true,
    };
    if (version === 'latest') {
        let newFeaturesSet = {
            sourceInformation: {
                type: 'github-repo',
                apiUri: `https://api.github.com/repos/${owner}/${repo}/releases/latest`,
                unauthenticatedUri: `https://github.com/${owner}/${repo}/releases/latest/download`,
                owner,
                repo,
                isLatest: true
            },
            features: [feat],
        };
        return newFeaturesSet;
    }
    else {
        // We must have a tag, return a tarball URI for the tagged version. 
        let newFeaturesSet = {
            sourceInformation: {
                type: 'github-repo',
                apiUri: `https://api.github.com/repos/${owner}/${repo}/releases/tags/${version}`,
                unauthenticatedUri: `https://github.com/${owner}/${repo}/releases/download/${version}`,
                owner,
                repo,
                tag: version,
                isLatest: false
            },
            features: [feat],
        };
        return newFeaturesSet;
    }
}
exports.parseFeatureIdentifier = parseFeatureIdentifier;
async function fetchFeatures(params, featuresConfig, localFeatures, dstFolder) {
    var _a, _b;
    for (const featureSet of featuresConfig.featureSets) {
        try {
            if (!featureSet || !featureSet.features || !featureSet.sourceInformation) {
                continue;
            }
            if (!localFeatures) {
                continue;
            }
            const feature = featureSet.features[0];
            const consecutiveId = feature.id + '_' + getCounter();
            // Calculate some predictable caching paths.
            const featCachePath = path.join(dstFolder, consecutiveId);
            const sourceInfoType = (_a = featureSet.sourceInformation) === null || _a === void 0 ? void 0 : _a.type;
            feature.infoString = featCachePath;
            feature.consecutiveId = consecutiveId;
            const featureDebugId = `${feature.consecutiveId}_${sourceInfoType}`;
            params.output.write(`* Fetching feature: ${featureDebugId}`);
            if (sourceInfoType === 'local-cache') {
                // create copy of the local features to set the environment variables for them.
                await (0, pfs_1.mkdirpLocal)(featCachePath);
                await (0, pfs_1.cpDirectoryLocal)(path.join(dstFolder, 'local-cache'), featCachePath);
                await parseDevContainerFeature(featureSet, feature, featCachePath);
                if (featureSet.internalVersion !== '2') {
                    const local = localFeatures.features.find(x => x.id === feature.id);
                    feature.buildArg = local === null || local === void 0 ? void 0 : local.buildArg;
                    feature.options = local === null || local === void 0 ? void 0 : local.options;
                }
                continue;
            }
            if (sourceInfoType === 'file-path') {
                params.output.write(`Detected local file path`, log_1.LogLevel.Trace);
                const executionPath = featureSet.sourceInformation.isRelative ? path.join(params.cwd, featureSet.sourceInformation.filePath) : featureSet.sourceInformation.filePath;
                await parseDevContainerFeature(featureSet, feature, executionPath);
                await (0, pfs_1.mkdirpLocal)(featCachePath);
                await (0, pfs_1.cpDirectoryLocal)(executionPath, featCachePath);
                continue;
            }
            params.output.write(`Detected tarball`, log_1.LogLevel.Trace);
            const headers = getRequestHeaders(featureSet.sourceInformation, params.env, params.output);
            // Ordered list of tarballUris to attempt to fetch from.
            let tarballUris = [];
            if (sourceInfoType === 'github-repo') {
                params.output.write('Determining tarball URI for provided github repo.', log_1.LogLevel.Trace);
                if (headers.Authorization && headers.Authorization !== '') {
                    params.output.write('GITHUB_TOKEN available. Attempting to fetch via GH API.', log_1.LogLevel.Info);
                    const authenticatedGithubTarballUri = await askGitHubApiForTarballUri(featureSet.sourceInformation, feature, headers, params.output);
                    if (authenticatedGithubTarballUri) {
                        tarballUris.push(authenticatedGithubTarballUri);
                    }
                    else {
                        params.output.write('Failed to generate autenticated tarball URI for provided feature, despite a GitHub token present', log_1.LogLevel.Warning);
                    }
                    headers.Accept = 'Accept: application/octet-stream';
                }
                // Always add the unauthenticated URIs as fallback options.
                params.output.write('Appending unauthenticated URIs for v2 and then v1', log_1.LogLevel.Trace);
                tarballUris.push(`${featureSet.sourceInformation.unauthenticatedUri}/${feature.id}.tgz`);
                tarballUris.push(`${featureSet.sourceInformation.unauthenticatedUri}/${V1_ASSET_NAME}`);
            }
            else {
                // We have a plain ol' tarball URI, since we aren't in the github-repo case.
                tarballUris.push(featureSet.sourceInformation.tarballUri);
            }
            // Attempt to fetch from 'tarballUris' in order, until one succeeds.
            let didSucceed = false;
            for (const tarballUri of tarballUris) {
                didSucceed = await fetchContentsAtTarballUri(tarballUri, featCachePath, headers, dstFolder, params.output);
                if (didSucceed) {
                    params.output.write(`Succeeded fetching ${tarballUri}`, log_1.LogLevel.Trace);
                    await parseDevContainerFeature(featureSet, feature, featCachePath);
                    break;
                }
            }
            if (!didSucceed) {
                const msg = `(!) Failed to fetch tarball for ${featureDebugId} after attempting ${tarballUris.length} possibilities.`;
                params.output.write(msg, log_1.LogLevel.Error);
                throw new Error(msg);
            }
        }
        catch (e) {
            params.output.write(`(!) ERR: Failed to fetch feature: ${(_b = e === null || e === void 0 ? void 0 : e.Message) !== null && _b !== void 0 ? _b : ''} `, log_1.LogLevel.Error);
            // TODO: Should this be more fatal?
        }
    }
}
async function fetchContentsAtTarballUri(tarballUri, featCachePath, headers, dstFolder, output) {
    const tempTarballPath = path.join(dstFolder, 'temp.tgz');
    try {
        const options = {
            type: 'GET',
            url: tarballUri,
            headers
        };
        output.write(`Fetching tarball at ${options.url}`);
        output.write(`Headers: ${JSON.stringify(options)}`, log_1.LogLevel.Trace);
        const tarball = await (0, httpRequest_1.request)(options, output);
        if (!tarball || tarball.length === 0) {
            output.write(`Did not receive a response from tarball download URI: ${tarballUri}`, log_1.LogLevel.Trace);
            return false;
        }
        // Filter what gets emitted from the tar.extract().
        const filter = (file, _) => {
            // Don't include .dotfiles or the archive itself.
            if (file.startsWith('./.') || file === `./${V1_ASSET_NAME}` || file === './.') {
                return false;
            }
            return true;
        };
        output.write(`Preparing to unarchive received tgz from ${tempTarballPath} -> ${featCachePath}.`, log_1.LogLevel.Trace);
        // Create the directory to cache this feature-set in.
        await (0, pfs_1.mkdirpLocal)(featCachePath);
        await (0, pfs_1.writeLocalFile)(tempTarballPath, tarball);
        await tar.x({
            file: tempTarballPath,
            cwd: featCachePath,
            filter
        });
        await cleanupIterationFetchAndMerge(tempTarballPath, output);
        return true;
    }
    catch (e) {
        output.write(`Caught failure when fetching from URI '${tarballUri}': ${e}`, log_1.LogLevel.Trace);
        await cleanupIterationFetchAndMerge(tempTarballPath, output);
        return false;
    }
}
async function parseDevContainerFeature(featureSet, feature, featCachePath) {
    var _a, _b;
    // Read version information.
    const jsonPath = path.join(featCachePath, 'devcontainer-feature.json');
    const innerPath = path.join(featCachePath, feature.id);
    const innerJsonPath = path.join(innerPath, 'devcontainer-feature.json');
    let foundPath;
    if ((0, fs_1.existsSync)(jsonPath)) {
        foundPath = jsonPath;
    }
    else if ((0, fs_1.existsSync)(innerJsonPath)) {
        foundPath = innerJsonPath;
        feature.infoString = innerPath;
    }
    if (foundPath) {
        const jsonString = await (0, pfs_1.readLocalFile)(foundPath);
        const featureJson = jsonc.parse(jsonString.toString());
        feature.runApp = '';
        feature.runParams = 'install.sh';
        if (featureJson.install) {
            feature.runApp = (_a = featureJson.install.app) !== null && _a !== void 0 ? _a : '';
            feature.runParams = (_b = featureJson.install.file) !== null && _b !== void 0 ? _b : 'install.sh';
        }
        feature.containerEnv = featureJson.containerEnv;
        featureSet.internalVersion = '2';
        feature.buildArg = featureJson.buildArg;
        feature.options = featureJson.options;
        feature.installAfter = featureJson.installAfter;
    }
    else {
        featureSet.internalVersion = '1';
    }
}
function getFeatureMainProperty(feature) {
    var _a;
    return ((_a = feature.options) === null || _a === void 0 ? void 0 : _a.version) ? 'version' : undefined;
}
exports.getFeatureMainProperty = getFeatureMainProperty;
function getFeatureMainValue(feature) {
    var _a, _b;
    const defaultProperty = getFeatureMainProperty(feature);
    if (!defaultProperty) {
        return !!feature.value;
    }
    if (typeof feature.value === 'object') {
        const value = feature.value[defaultProperty];
        if (value === undefined && feature.options) {
            return (_a = feature.options[defaultProperty]) === null || _a === void 0 ? void 0 : _a.default;
        }
        return value;
    }
    if (feature.value === undefined && feature.options) {
        return (_b = feature.options[defaultProperty]) === null || _b === void 0 ? void 0 : _b.default;
    }
    return feature.value;
}
exports.getFeatureMainValue = getFeatureMainValue;
function getFeatureValueObject(feature) {
    if (typeof feature.value === 'object') {
        return {
            ...getFeatureValueDefaults(feature),
            ...feature.value
        };
    }
    const mainProperty = getFeatureMainProperty(feature);
    if (!mainProperty) {
        return getFeatureValueDefaults(feature);
    }
    return {
        ...getFeatureValueDefaults(feature),
        [mainProperty]: feature.value,
    };
}
exports.getFeatureValueObject = getFeatureValueObject;
function getFeatureValueDefaults(feature) {
    const options = feature.options || {};
    return Object.keys(options)
        .reduce((defaults, key) => {
        if ('default' in options[key]) {
            defaults[key] = options[key].default;
        }
        return defaults;
    }, {});
}
//# sourceMappingURL=containerFeaturesConfiguration.js.map