import yargs, { Argv } from 'yargs';
import { ProvisionOptions } from './devContainers';
import { LogLevel } from '../spec-utils/log';
import { UnpackPromise } from '../spec-utils/types';
import { UserEnvProbe } from '../spec-common/injectHeadless';
import { CLIHost } from '../spec-common/cliHost';
import { PackageConfiguration } from '../spec-utils/product';
declare type UnpackArgv<T> = T extends Argv<infer U> ? U : T;
declare function doProvision(options: ProvisionOptions): Promise<{
    containerId: string;
    composeProjectName: string | undefined;
    remoteUser: string;
    remoteWorkspaceFolder: string | undefined;
    finishBackgroundTasks: () => Promise<void>;
    outcome: "success";
    dispose: () => Promise<void>;
    message?: undefined;
    description?: undefined;
} | {
    outcome: "error";
    message: string;
    description: string;
    containerId: string | undefined;
    dispose: () => Promise<void>;
}>;
export declare type Result = UnpackPromise<ReturnType<typeof doProvision>> & {
    backgroundProcessPID?: number;
};
declare function execOptions(y: Argv): yargs.Argv<yargs.Omit<{}, "config" | "user-data-folder" | "docker-path" | "docker-compose-path" | "container-data-folder" | "container-system-data-folder" | "workspace-folder" | "mount-workspace-git-root" | "container-id" | "id-label" | "override-config" | "log-level" | "log-format" | "terminal-columns" | "terminal-rows" | "default-user-env-probe" | "remote-env"> & yargs.InferredOptionTypes<{
    'user-data-folder': {
        type: "string";
        description: string;
    };
    'docker-path': {
        type: "string";
        description: string;
    };
    'docker-compose-path': {
        type: "string";
        description: string;
    };
    'container-data-folder': {
        type: "string";
        description: string;
    };
    'container-system-data-folder': {
        type: "string";
        description: string;
    };
    'workspace-folder': {
        type: "string";
        required: true;
        description: string;
    };
    'mount-workspace-git-root': {
        type: "boolean";
        default: boolean;
        description: string;
    };
    'container-id': {
        type: "string";
        description: string;
    };
    'id-label': {
        type: "string";
        description: string;
    };
    config: {
        type: "string";
        description: string;
    };
    'override-config': {
        type: "string";
        description: string;
    };
    'log-level': {
        choices: ("info" | "debug" | "trace")[];
        default: "info";
        description: string;
    };
    'log-format': {
        choices: ("json" | "text")[];
        default: "text";
        description: string;
    };
    'terminal-columns': {
        type: "number";
        implies: string[];
        description: string;
    };
    'terminal-rows': {
        type: "number";
        implies: string[];
        description: string;
    };
    'default-user-env-probe': {
        choices: ("none" | "loginInteractiveShell" | "interactiveShell" | "loginShell")[];
        default: UserEnvProbe;
        description: string;
    };
    'remote-env': {
        type: "string";
        description: string;
    };
}> & {
    cmd: string;
} & {
    args: string[];
}>;
export declare type ExecArgs = UnpackArgv<ReturnType<typeof execOptions>>;
export declare function doExec({ 'user-data-folder': persistedFolder, 'docker-path': dockerPath, 'docker-compose-path': dockerComposePath, 'container-data-folder': containerDataFolder, 'container-system-data-folder': containerSystemDataFolder, 'workspace-folder': workspaceFolderArg, 'mount-workspace-git-root': mountWorkspaceGitRoot, 'container-id': containerId, 'id-label': idLabel, config: configParam, 'override-config': overrideConfig, 'log-level': logLevel, 'log-format': logFormat, 'terminal-rows': terminalRows, 'terminal-columns': terminalColumns, 'default-user-env-probe': defaultUserEnvProbe, 'remote-env': addRemoteEnv, _: restArgs, }: ExecArgs & {
    _?: string[];
}): Promise<{
    outcome: "success";
    dispose: () => Promise<void>;
    message?: undefined;
    description?: undefined;
    containerId?: undefined;
} | {
    outcome: "error";
    message: string;
    description: string;
    containerId: string | undefined;
    dispose: () => Promise<void>;
}>;
declare function featuresTestOptions(y: Argv): yargs.Argv<yargs.Omit<{}, "features" | "log-level" | "quiet" | "base-image" | "scenarios" | "remote-user" | "collection-folder"> & yargs.InferredOptionTypes<{
    'base-image': {
        type: "string";
        alias: string;
        default: string;
        description: string;
    };
    features: {
        type: "array";
        alias: string;
        describe: string;
    };
    scenarios: {
        type: "string";
        alias: string;
        description: string;
    };
    'remote-user': {
        type: "string";
        alias: string;
        default: string;
        describe: string;
    };
    'collection-folder': {
        type: "string";
        alias: string;
        default: string;
        description: string;
    };
    'log-level': {
        choices: ("info" | "debug" | "trace")[];
        default: "info";
        description: string;
    };
    quiet: {
        type: "boolean";
        alias: string;
        default: boolean;
        description: string;
    };
}>>;
export declare type FeaturesTestArgs = UnpackArgv<ReturnType<typeof featuresTestOptions>>;
export interface FeaturesTestCommandInput {
    cliHost: CLIHost;
    pkg: PackageConfiguration;
    baseImage: string;
    collectionFolder: string;
    features?: string[];
    scenariosFolder: string | undefined;
    remoteUser: string;
    quiet: boolean;
    logLevel: LogLevel;
    disposables: (() => Promise<unknown> | undefined)[];
}
export {};
